﻿/*
 * ----------------------------------------------------------------------------
 *
 *  Gator News Ticker for Google Desktop Sidebar.
 *
 *  Copyright (c) 2006, Brennan Folmer
 *  http://brennanfolmer.com/GatorNewsTicker/
 *
 *  Changes:
 *    2006-08-05: Initial version
 * ----------------------------------------------------------------------------
 */


// All categories enabled by default
options.defaultValue("Audio") = "true";
options.defaultValue("Campus") = "true";
options.defaultValue("General") = "true";
options.defaultValue("Op-Eds") = "true";
options.defaultValue("Research") = "true";
options.defaultValue("UF In The News") = "true";
options.defaultValue("Sports") = "true";

var menuItems = new Array("Audio","Campus","General","Op-Eds","Research","UF In The News","Sports");

var strAboutText = "Gator News Ticker\n&copy; 2006 Brennan Folmer\nThis Google Gadget retrieves and displays news articles from the University of Florida. To filter articles by topic, select or de-select topic names on the menu.\n\nhttp://brennanfolmer.com/GatorNewsTicker/\n\nbrennanfolmer@gmail.com";

var articleMaxAge = 14;  // Max age of articles, in days

// Some basic settings for the plugin
plugin.onAddCustomMenuItems = AddCustomMenuItems;  // handler to add menu items
plugin.about_text = strAboutText;
view.caption = GADGET_NAME;
view.showCaptionAlways = true;
cnArea.contentFlags = gddContentFlagHaveDetails;

setInterval(OnTimer, 1000 * 30);  // check for new content once every 30 seconds
setInterval(Cleanup, 1000 * 60 * 60 * 24);  // check once per day for old items

OnTimer();    // update with first set of items

function AddCustomMenuItems(menu) {
  // Add the various item layouts as menu items and check the currently selected one
  for (var i = 0; i < menuItems.length; ++i) {
    if (options(menuItems[i]) == "true") {
      menu.AddItem(menuItems[i], gddMenuItemFlagChecked, MenuItemClicked);
    } else {
      menu.AddItem(menuItems[i], 0, MenuItemClicked);
    }
  }    
}

function MenuItemClicked(item) {
  // One of the filters was clicked
  if (options(item) == "true")
  {  
     // Removing the category
     options(item) = "false";
     var vbarrItems = cnArea.contentItems;
     if (vbarrItems != null) 
     {
        var items = vbarrItems.toArray();
        for (var j = 0; j < items.length; j++) 
        {
          if (items[j].source == item)
          {
             cnArea.removeContentItem(items[j]);
          }
        }
     }
  }
  else
  { // Adding the category
     options(item) = "true";
     OnTimer();
  }
}

function OnTimer() {
  // Update from both the news site and the sports site
  var item = new Update("http://news.ufl.edu/feed/");
  item = null;
  item = new Update("http://gatorzone.com/rss/");
  item = null;
}

// Manage the requests and callbacks of an item
function Update(url) {
  // send request to webserver for data
  var http = new XMLHttpRequest();
  http.onreadystatechange = OnData;
  http.open("GET", url, true);
  http.send();

  function OnData() 
  {
    if (http.readyState == 4) { // completed?
      // parse the xml data received
      var doc = new DOMDocument();
      doc.loadXML(http.responseText);

      // read the word title, description and link from the xml
      var title, desc, link, category, audioLink, pubDate;

      var elem = doc.getElementsByTagName("item");
      if (elem != null && elem.length > 0) 
      {
       // Run through the XML
       for (var i = 0; i < elem.length; i++)
       {
        category = "";
        title = "";
        desc = "";
        link = "";
        audioLink = "";
        pubDate = "";

        for (var node = elem[i].firstChild; node != null; node = node.nextSibling) 
        {
          if (node.nodeName == "title")
            title = node.firstChild.nodeValue;
          //if (node.nodeName == "content:encoded")
          //  content = node.firstChild.nodeValue;
          if (node.nodeName == "pubDate")
            pubDate = node.firstChild.nodeValue;
          if (node.nodeName == "description")
            desc = node.firstChild.nodeValue;
          if (node.nodeName == "link")
            link = node.firstChild.nodeValue;
	  if (node.nodeName == "category" && category == "")
            category = node.firstChild.nodeValue;
	  if (node.nodeName == "enclosure")
            audioLink = node.attributes[0].value;
        }

        desc = desc.replace(/<([^>]|\n)*>/g,'');  // remove html tags if present
        desc = desc.replace(/&nbsp;/g, ' ');
        desc = desc.replace(/&quot;/g, '"');
        desc = desc.replace(/&amp;/g, '&');
        desc = desc.replace(/&#39;/g, '\'');
        link = link.replace(/&amp;/g, '&');
        link = link.replace(/&#39;/g, '\'');
        title = title.replace(/&amp;/g, '&');
        title = title.replace(/&#39;/g, '\'');

        // There is not a 'category' node on the sports feed
        if (url == "http://gatorzone.com/rss/")
          category = "Sports";
        
        if (title != null) 
        {  
           var vbarrItems = cnArea.contentItems;
           if (vbarrItems != null) 
           {
             var items = vbarrItems.toArray();
             for (var j = 0; j < items.length; j++) 
             {
               if (items[j].open_command == link || items[j].open_command == audioLink)
                 break;
             }

             var curItem = null;
             if (j == items.length) 
             {
               curItem = new ContentItem();  // item does not already exist
               curItem.source = category;
               curItem.layout = gddContentItemLayoutNews;
               curItem.flags = gddContentItemFlagShareable | gddContentItemFlagNoRemove;
               curItem.onDetailsView = OnDetailsView;

               curItem.heading = title;
               curItem.snippet = desc;     // to show in details view (for Audio)

               var tm = new Date();
               var date_string = Date.parse(pubDate);
               tm.setTime(date_string);
               tm.setMinutes(tm.getMinutes() + tm.getTimezoneOffset());
               curItem.time_created = tm.getVarDate();

               // If this is audio, fill from the 'enclosure' link
               if (category == "Audio")
               {
                  curItem.open_command = audioLink;
               } 
               else
               {
                  curItem.open_command = link;
               }

               // Only add items from categories that are not filtered and that are not older than articleMaxAge days
               var dtMaxAge = new Date();
               dtMaxAge.setDate(dtMaxAge.getDate() - articleMaxAge);
               if (options(category) == "true" && curItem.time_created > dtMaxAge)
                   cnArea.addContentItem(curItem, gddItemDisplayInSidebar|gddItemDisplayAsNotificationIfSidebarHidden);
             }
           }
        }
       } // end for
      } // end if
      http = null;  // free the http object
    } //end if
  } //end OnData
}

function OnDetailsView(item) {

  if (item.source == "Audio")
  { 
          // The sound does not play consistently in the web browser control.... present link
	  var html_text = "<head><style type=\"text/css\">  body {font-size: 80%;font-family: 'Lucida Grande', Verdana, Arial, Sans-Serif, \"Times New Roman\",Times, serif;}</style></head><body>" + item.snippet + "<br><br><a href=\"" + item.open_command + "\">Click here for audio</a></body>";
	  var control = new DetailsView();
          var tm = new Date();
          var date_string = Date.parse(item.time_created);
          tm.setTime(date_string);
          tm.setMinutes(tm.getMinutes() + tm.getTimezoneOffset());
	  control.SetContent("University of Florida :: " + item.source, tm.getVarDate(), html_text,
	    true, item.layout);
	  control.html_content = true;

	  var details = new Object();
	  details.details_control = control;
	  
	  return details;
  } 
  else
  {
	  var obj = new Object();

	  // Create the Internet Explorer object and return it
	  IEControl = new WebBrowser();
	  obj.details_control = IEControl;

	  // Fill in the URL that will be the target for this navigation
	  options.defaultValue("navurl") = item.open_command;

	  // After a short interval, navigate to the proper webpage
	  IETimer = utils.setInterval(StartNavigation, 100);

	  return obj; 
  }
}

function StartNavigation() {

  // Do not let the timer restart
  utils.clearInterval(IETimer);

  // Navigate to the news page
  IEControl.navigate(options("navurl")); 
}

function Cleanup() {
  
  // Run through the items and remove those that are older than articleMaxAge days
  var vbarrItems = cnArea.contentItems;
  if (vbarrItems != null) 
  {
     var items = vbarrItems.toArray();
     var dtMaxAge = new Date();
     dtMaxAge.setDate(dtMaxAge.getDate() - articleMaxAge);
     for (var j = 0; j < items.length; j++) 
     {
       if (items[j].time_created <= dtMaxAge)
         cnArea.removeContentItem(items[j]);
     }
  }
}
